#include "image.h"

#include <QDir>
#include <QDebug>
#include <QDate>
#include <QImageWriter>
#include <QImageReader>


CImage::CImage(QObject *parent) :
    QObject(parent)
{
    m_imageNumber = 1;
    m_imageChanged = false;
    m_dirPath = QDir::homePath() + "/QScanner";
    m_namePattern = "image";
    m_extension = "png";
}

CImage::~CImage()
{
    qDebug() << "saving......." << m_filePath;
    saveImage();
}

void CImage::setSettings(QList<QVariant> sets)
{
    //1. dir Name, 2. extension, 3. baseName, 4.start nr
    m_dirPath = sets.at(0).toString();
    m_extension = sets.at(1).toString();
    m_namePattern = sets.at(2).toString();
    m_imageNumber = sets.at(3).toInt();

}

bool CImage::addImage(QImage image)
{
    saveImage();
    //create new one
    m_mutex.lock();
    m_pixmap = QPixmap::fromImage(image);
    m_filePath = createNewFullFileName();
    bool ok = image.save(m_filePath);
    if (!ok) {
        qDebug() << "CImage::addImage - Image not saved!" << m_filePath;
        m_mutex.unlock();
        return false;
    }
    //emit signal for imageViewer, you can show image and thumbnail
    emit imageReady(m_pixmap, thumbnail(), imageInfo());
    m_mutex.unlock();
    return true;
}

bool CImage::addImage(QByteArray &image, int width, int height, int bytespl, int format)
{
    //maybe it would be better to use QByteArray rather then QPixmap
    return false;
}

bool CImage::addImage(const QString filePath)
{
    QFile file(filePath);
    if (!file.exists())
        return false;

    //m_mutex.lock();
    m_filePath = filePath;
    m_pixmap = QPixmap(filePath);
    if (m_pixmap.isNull())
        //m_mutex.unlock();
        return false;

    //emit signal for imageViewer, you can show image and thumbnail
    emit imageReady(m_pixmap, thumbnail(), imageInfo());
    //m_mutex.unlock();
    return true;
}

void CImage::setCurrentImage(const QString filePath)
{
    saveImage();
    // set new one
    m_pixmap = QPixmap(filePath);
    m_filePath = filePath;
    emit imageOpened(m_pixmap, imageInfo());
}

QPixmap &CImage::currentImage()
{
    return m_pixmap;
}

QPixmap &CImage::rotateLeft()
{
    QTransform transform;
    transform.rotate(-90);
    m_pixmap = m_pixmap.transformed(transform);
    m_imageChanged = true;
    return m_pixmap;
}

QPixmap &CImage::rotateRight()
{
    QTransform transform;
    transform.rotate(90);
    m_pixmap = m_pixmap.transformed(transform);
    m_imageChanged = true;
    return m_pixmap;
}

QPixmap &CImage::flipHorizontally()
{
    QTransform transform;
    transform.scale(-1,1);
    m_pixmap = m_pixmap.transformed(transform);
    m_imageChanged = true;
    return m_pixmap;
}

QPixmap &CImage::flipVertiaclly()
{
    QTransform transform;
    transform.scale(1,-1);
    m_pixmap = m_pixmap.transformed(transform);
    m_imageChanged = true;
    return m_pixmap;
}

QPixmap &CImage::scale(QSize size)
{
    m_pixmap = m_pixmap.scaled(size,Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
    m_imageChanged = true;
    saveImage();
    return m_pixmap;
}

QIcon CImage::thumbnail()
{
    return QIcon(m_pixmap.scaled(QSize(80, 120), Qt::KeepAspectRatio));
}

QString CImage::filePath()
{
    return m_filePath;
}

QString CImage::imageInfo()
{
    estimateImageSize();
    QString info = m_filePath + "\t" + m_imageSize + "\t" + m_imageDim;
    return info;
}

QString CImage::createNewFullFileName()
{
    QString fileName;
    QFile file;
    do {
        fileName = m_dirPath + "/" + m_namePattern + "-" + QString::number(m_imageNumber++) + "." + m_extension;
        file.setFileName(fileName);
    } while (file.exists());
    qDebug() << "FileName" << fileName;
    return fileName;
}

void CImage::saveImage()
{
    if (m_pixmap.isNull())
        return;
    m_mutex.lock();
    if (m_imageChanged)
    {
        m_pixmap.save(m_filePath);
        m_imageChanged = false;
    }
    m_mutex.unlock();
}

void CImage::saveImageAs(QString oldFilePath, QString newFilePath, QString extension, int quality)
{
    QImageReader reader(oldFilePath);

    QImageWriter writer(newFilePath, extension.toLocal8Bit());
    if (quality != -1) {
        writer.setQuality(10);
    }
    writer.write(reader.read());
}

void CImage::setVoidImage()
{
    m_pixmap = QPixmap();
}

void CImage::deleteFile(QString fileName)
{
    QFile file(fileName);
    if (file.remove())
        qDebug() << fileName << "deleted from disc.";
    else
        qDebug() << fileName << "not deleted from disc.";
}

void CImage::estimateImageSize()
{
    if (m_filePath.isEmpty())
        return;

    QFileInfo fileInfo(m_filePath);
    double size;
    if (fileInfo.size() < 1048576) {
        size= (double)fileInfo.size() / 1024;
        m_imageSize = QString(" %1 kiB ").arg(size,0,'f',0);
    } else {
        size= (double)fileInfo.size() / 1048576;
        m_imageSize = QString(" %1 MiB ").arg(size,0,'f',1);
    }

    m_imageDim = tr(" %1 x %2 ").arg(m_pixmap.size().width()).arg(m_pixmap.size().height());
}
