#include "imageviewer.h"
#include "ui_imageviewer.h"
#include "scaleimage.h"
#include "dialogs/savedialog.h"
#include "image.h"

#include <QDebug>
#include <QFileDialog>
#include <QMessageBox>
#include <QPushButton>
#include <QPrintDialog>
#include <QPrinter>
#include <QPainter>
#include <QListWidgetItem>
#include <QDate>
#include <QCloseEvent>
#include <QImageWriter>
#include <QToolTip>

CImageViewer::CImageViewer(CImage *image, QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::CImageViewer)
{
    ui->setupUi(this);
    m_image = image;
    connect(m_image, SIGNAL(imageReady(QPixmap&, QIcon, QString)), this, SLOT(newImageReady(QPixmap &, QIcon, QString)));
    connect(m_image, SIGNAL(imageOpened(QPixmap&,QString)), this, SLOT(newImageReady(QPixmap&,QString)));

    ui->bSave->setDefaultAction(ui->actionSave);
    ui->bPrint->setDefaultAction(ui->actionPrint);
    ui->bSaveAsPdf->setDefaultAction(ui->actionSave_as_pdf);
    ui->bUndo->setDefaultAction(ui->actionUndo);
    ui->bUndo->hide();  //unused currently
    ui->bRotateLeft->setDefaultAction(ui->actionRotate_left);
    ui->bRotateRight->setDefaultAction(ui->actionRotate_right);
    ui->bFlipHor->setDefaultAction(ui->actionFlip_horizontally);
    ui->bFlipVert->setDefaultAction(ui->actionFlip_vertical);
    ui->bScaleImage->setDefaultAction(ui->actionScale_image);

    ui->actionShow_images_thumbnail->setChecked(true);
    ui->bShowList->setDefaultAction(ui->actionShow_images_thumbnail);
    ui->bZoomFit->setDefaultAction(ui->actionZoom_to_fit);
    ui->BZoom100->setDefaultAction(ui->actionZoom_to_100);
    ui->bZoomIn->setDefaultAction(ui->actionZoom_in);
    ui->bZoomOut->setDefaultAction(ui->actionZoom_out);

    enableActions(false);

    connect(ui->listImages, SIGNAL(customItemClicked(QListWidgetItem*,QListWidgetItem*)), this, SLOT(changeImageOnListClicked(QListWidgetItem*,QListWidgetItem*)));
    connect(ui->listImages, SIGNAL(deleteFile(QString)), this, SLOT(deleteFile(QString)));
    connect(ui->listImages, SIGNAL(listIsEmpty()), this, SLOT(listIsEmpty()));

}

CImageViewer::~CImageViewer()
{
    qDebug() << "~~~destr ImageViewer";
    delete ui;
}

QStringList CImageViewer::openedImages()
{
    return ui->listImages->fileList();
}

void CImageViewer::newImageReady(QPixmap &image, QIcon icon, QString info)
{
    //show image
    ui->imageWidget->setImage(image);
    if (ui->bZoomFit->isChecked()) {
        ui->imageWidget->setFitInView(true);
    }

    //add new item to the list
    if (ui->listImages->count() == 0) {
        enableActions(true);
    }
    QListWidgetItem *item = new QListWidgetItem(icon,"");
    item->setData(Qt::UserRole, m_image->filePath());
    item->setToolTip(m_image->filePath());
    ui->listImages->addItem(item);
    ui->listImages->setCurrentItem(item);
    updateLabelInfo(info);
}

void CImageViewer::newImageReady(QPixmap &image, QString info)
{
    //show image
    ui->imageWidget->setImage(image);
    if (ui->bZoomFit->isChecked()) {
        ui->imageWidget->setFitInView(true);
    }
    updateLabelInfo(info);
}

void CImageViewer::closeEvent(QCloseEvent *event)
{
    QMainWindow::closeEvent(event);
}

void CImageViewer::on_actionSave_triggered()
{
    m_image->saveImage();

    CSaveDialog *dialog = new CSaveDialog(this);
    dialog->setModal(true);
    if (!dialog->exec()) {
        return;
    }
    if (dialog->saveAllImages()) {
        for (int i=0; i<ui->listImages->count(); i++) {
            QString oldName = ui->listImages->item(i)->data(Qt::UserRole).toString();
            QString newName = dialog->fullFilePath() + QString::number(i+1) + "." + dialog->extension();
            if (fileExist(oldName)){
                overwriteOrRenameImage(oldName, newName, dialog->extension());

            } else {
                m_image->saveImageAs(oldName, newName, dialog->extension());
                qDebug() <<"Image saved" << oldName << newName;
            }
        }
    } else {  //save one image only
        QString oldName = ui->listImages->currentItemFileName();
        QString newName = dialog->fullFilePath() + "." + dialog->extension();
        m_image->saveImageAs(oldName, newName, dialog->extension());
        qDebug() <<"Image saved" << oldName << newName;
    }
}

void CImageViewer::on_actionSave_as_pdf_triggered()
{
    m_image->saveImage();

    QString fileName = QFileDialog::getSaveFileName(this, tr("Save images as pdf"), QDir::homePath(), tr("Pdf files (*.pdf)"));
    if (fileName.isEmpty())
        return;
    QPrinter printer;
    printer.setOutputFormat(QPrinter::PdfFormat);
    printer.setOutputFileName(fileName);
    printer.setPaperSize(QPrinter::A4);
    printer.setColorMode(QPrinter::Color);
    QPainter painter(&printer);
    QRect rect = painter.viewport();
    QListWidgetItem *item;
    for (int i=0; i<ui->listImages->count(); i++) {
        if (ui->listImages->item(i)->isHidden()) {
            continue;
        }
        item = ui->listImages->item(i);
        QImage img(item->data(Qt::UserRole).toString());
        scaleAndFitImage(img, rect);
        painter.drawImage(0,0, img);
        if (i!= ui->listImages->count()-1)
            printer.newPage();
    }
    painter.end();
}

void CImageViewer::scaleAndFitImage(QImage &img, const QRect rect)
{
    //1. Image is smaller then page
    if (rect.height() > img.height() && rect.width() > img.width()) {
        return;
    }
    //2. choose orientation - vert / horiz
    if (img.width() > img.height()) {
        QTransform transform;
        transform.rotate(90);
        img = img.transformed(transform);
    }
    //3. scale to best fit to page
    img = img.scaled(rect.size(), Qt::KeepAspectRatio, Qt::SmoothTransformation);
}

void CImageViewer::on_actionPrint_triggered()
{
    m_image->saveImage();

    QPrinter printer;
    printer.setColorMode(QPrinter::Color);
    QPrintDialog dlg(&printer,0);
    if(dlg.exec() == QDialog::Accepted) {
        QPainter painter(&printer);
        QRect rect = painter.viewport();
        QPixmap img = m_image->currentImage().scaled(rect.size(), Qt::KeepAspectRatio, Qt::SmoothTransformation);
        painter.drawPixmap(0,0, img);
        painter.end();
    }
}

void CImageViewer::on_actionUndo_triggered()
{
    ui->imageWidget->clear(); //TODO restore original image
}

void CImageViewer::on_actionRotate_left_triggered()
{
    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    ui->imageWidget->setImage(m_image->rotateLeft());
    ui->listImages->rotateThumbnailLeft();
    QApplication::restoreOverrideCursor();
}

void CImageViewer::on_actionRotate_right_triggered()
{
    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    ui->imageWidget->setImage(m_image->rotateRight());
    ui->listImages->rotateThumbnailRight();
    QApplication::restoreOverrideCursor();
}

void CImageViewer::on_actionFlip_horizontally_triggered()
{
    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    ui->imageWidget->setImage(m_image->flipHorizontally());
    ui->listImages->flipThumbnailHorizontaly();
    QApplication::restoreOverrideCursor();
}

void CImageViewer::on_actionFlip_vertical_triggered()
{
    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    ui->imageWidget->setImage(m_image->flipVertiaclly());
    ui->listImages->flipThumbnailVerticaly();
    QApplication::restoreOverrideCursor();
}

void CImageViewer::on_actionScale_image_triggered()
{
    //just in any case checking
    if (!m_image->currentImage().isNull()) {
        CScaleImage *window = new CScaleImage(QCursor::pos(), m_image->currentImage().size());
        connect(window, SIGNAL(newSize(QSize)), this, SLOT(scaleMainImage(QSize)));
        window->show();
    }
}

void CImageViewer::scaleMainImage(QSize newSize)
{
    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    ui->imageWidget->setImage(m_image->scale(newSize));
    updateLabelInfo(m_image->imageInfo());
    QApplication::restoreOverrideCursor();
}

void CImageViewer::on_actionZoom_to_fit_triggered()
{
    ui->imageWidget->setFitInView(true);
    ui->actionZoom_to_100->setChecked(false);
    updateZoomFactor();
}

void CImageViewer::on_actionZoom_to_100_triggered()
{
    ui->imageWidget->setFitInView(false);
    ui->actionZoom_to_fit->setChecked(false);
    updateZoomFactor();
}

void CImageViewer::on_actionZoom_in_triggered()
{
    ui->actionZoom_to_fit->setChecked(false);
    ui->actionZoom_to_100->setChecked(false);
    ui->imageWidget->setZoomIn();
    updateZoomFactor();
}

void CImageViewer::on_actionZoom_out_triggered()
{
    ui->actionZoom_to_fit->setChecked(false);
    ui->actionZoom_to_100->setChecked(false);
    ui->imageWidget->setZoomOut();
    updateZoomFactor();
}

void CImageViewer::deleteFile(QString filePath)
{
    QMessageBox::StandardButton ret = QMessageBox::question(this, tr("Delete file"), tr("Are you sure you want to delete this file permanently from disc?"), QMessageBox::Yes | QMessageBox::No, QMessageBox::No);
    if (ret == QMessageBox::Yes) {
        m_image->deleteFile(filePath);
        ui->listImages->removeItemFromList();
        changeImageOnListClicked(new QListWidgetItem(), ui->listImages->currentItem());
    }
}

void CImageViewer::listIsEmpty()
{
    ui->imageWidget->clear();
    m_image->setVoidImage();
    enableActions(false);
}

void CImageViewer::enableActions(bool enable)
{
    ui->actionFlip_horizontally->setEnabled(enable);
    ui->actionFlip_vertical->setEnabled(enable);
    ui->actionPrint->setEnabled(enable);
    ui->actionRotate_left->setEnabled(enable);
    ui->actionRotate_right->setEnabled(enable);
    ui->actionSave->setEnabled(enable);
    ui->actionSave_as_pdf->setEnabled(enable);
    ui->actionScale_image->setEnabled(enable);
}

void CImageViewer::on_actionShow_images_thumbnail_triggered(bool checked)
{
    ui->listImages->setVisible(checked);
}

void CImageViewer::changeImageOnListClicked(QListWidgetItem *itemOld,QListWidgetItem*itemNew)
{
    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    //check if list isn't empty
    if (!itemNew) {
        ui->imageWidget->clear();
        QApplication::restoreOverrideCursor();
        return;
    }
    //read new image
    m_image->setCurrentImage(itemNew->data(Qt::UserRole).toString());
    QApplication::restoreOverrideCursor();
}

void CImageViewer::updateLabelInfo(QString fileInfo)
{
    ui->imageWidget->setImageInfo(fileInfo);
}

void CImageViewer::updateZoomFactor()
{
    int zoom = ui->imageWidget->zoomFactor();
    if (zoom == 100 && !ui->actionZoom_to_100->isChecked()) {
       ui->actionZoom_to_100->setChecked(true);
    }
    QString zoomStr = QString::number(zoom) + QString(" %");
    ui->zoomText->setText(zoomStr);
}

bool CImageViewer::fileExist(QString fileName)
{
    QFile file(fileName);
    return file.exists();

}

void CImageViewer::overwriteOrRenameImage(QString oldFilePath, QString newFilePath, QString extension)
{
    //TODO more advanced dialog: yes to all, etc
    QMessageBox msgBox;
    msgBox.setWindowTitle(tr("Owerwrite file?"));
    msgBox.setText(tr("The file ") + newFilePath + tr(" already exists. \nDo you wish to overwrite it?"));
    QPushButton *overwriteB = msgBox.addButton(tr("Overwrite"), QMessageBox::AcceptRole);
    QPushButton *changeNameB = msgBox.addButton(tr("Change name"), QMessageBox::ActionRole);
    msgBox.addButton(QMessageBox::Discard);
    msgBox.exec();
    if (msgBox.clickedButton() == overwriteB) {
        m_image->saveImageAs(oldFilePath, newFilePath, extension);
        qDebug() <<"Image saved" << oldFilePath;
    }
    if (msgBox.clickedButton() == changeNameB) {
        //split filename to insert _number before .extension
        QString fileName;
        QFileInfo info(newFilePath);
        QString file = info.absolutePath() + "/" + info.baseName();
        //find first free number
        int lp = 1;
        do {
            fileName = file + "_" + QString::number(lp) + "." + info.completeSuffix();
            lp++;
        } while (fileExist((fileName)));
        m_image->saveImageAs(oldFilePath, fileName, extension);
        qDebug() <<"Image saved" << oldFilePath << fileName ;
    }
}
