#include "imagewidget.h"

#include <QPainter>
#include <QDebug>
#include <QToolTip>
#include <QApplication>
#include <QStyleOption>

const double ZOOM = 1.25;

ImageWidget::ImageWidget(QWidget *parent) :
    QGraphicsView(parent)
{
    m_scene = new QGraphicsScene;
    setScene(m_scene);
    setDragMode(ScrollHandDrag);
    setRenderHints(QPainter::Antialiasing | QPainter::SmoothPixmapTransform);
    m_zoomFactor = 1;
    setViewportUpdateMode(QGraphicsView::BoundingRectViewportUpdate);
}

void ImageWidget::setImage(QPixmap &image)
{
    if (image.isNull())
        return;

    m_image = image;
    m_scene->setSceneRect(0, 0, image.width(), image.height());
    resetCachedContent();
    update();
    viewport()->update();
}

int ImageWidget::zoomFactor()
{
    int zoom = (int) (m_zoomFactor*100);
    return zoom;
}

void ImageWidget::drawBackground(QPainter *painter, const QRectF &rect)
{
    painter->fillRect(rect, QColor(Qt::darkGray));
    painter->drawPixmap(rect, m_image, rect);
}

void ImageWidget::paintEvent(QPaintEvent *event)
{
    QGraphicsView::paintEvent(event);

    //draw imageInfo rect only when necessery
    if (!m_imageInfo.isEmpty()) {
        QPainter painter;
        painter.begin(viewport() );
        //rect to paint
        QFontMetrics fm(QToolTip::font());
        int fontH = fm.height()+2;
        int fontW = fm.width(m_imageInfo);
        QRect rect;
        rect.setX(0);
        rect.setY(viewport()->rect().height() - fontH);
        rect.setWidth(fontW);
        rect.setHeight(fontH);
        //draw tooltip rect
        const QStyle *st = QApplication::style();
        QStyleOption opt;
        opt.rect = rect;
        st->drawPrimitive(QStyle::PE_PanelTipLabel, &opt, &painter);

        //draw tooltip text;
        st->drawItemText(&painter, rect, Qt::AlignCenter, palette(), true, m_imageInfo, QPalette::ToolTipText );
        painter.end();
    }
}

void ImageWidget::resizeEvent(QResizeEvent *event)
{
    QGraphicsView::resizeEvent(event);
    setFitInView(m_fitInView);
}

void ImageWidget::setZoomIn()
{
    scale(ZOOM, ZOOM);
    m_zoomFactor = ZOOM * m_zoomFactor;
}

void ImageWidget::setZoomOut()
{
    scale(1.0 / ZOOM, 1.0 / ZOOM);
    m_zoomFactor = m_zoomFactor /ZOOM;
}

void ImageWidget::setFitInView(bool fit)
{
    m_fitInView = fit;
    //do not zoom-in smaller images
    if (m_image.height() < height() && m_image.width() < width())
        return;

    if (fit) {
        fitInView(m_image.rect(), Qt::KeepAspectRatio);
        m_zoomFactor = height() / m_scene->height();
    } else {
        setMatrix(QMatrix());
        m_zoomFactor = 1;
    }
    resetCachedContent();
}

void ImageWidget::clear()
{
    m_imageInfo.clear();
    m_image = QPixmap();
    m_scene->setSceneRect(0, 0, width(), height());
    resetCachedContent();
}
