#include "mainwindow.h"
#include "ui_mainwindow.h"
#include"dialogs/settingsdialog.h"

#include <QDebug>
#include <QDir>
#include <QSettings>
#include <QCloseEvent>
#include <QMessageBox>
#include <libksane/ksane.h>
#include <sane/sane.h>


MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    m_image = new CImage;
    m_imageThread = new QThread;
    m_image->moveToThread(m_imageThread);
    m_imageThread->start();

    qDebug() << "Inicializing libKSane...";
    m_ksane = new KSaneIface::KSaneWidget();
    ui->tabWidget->addTab(m_ksane,tr("Scan"));
    connect(m_ksane,SIGNAL(imageReady(QByteArray &, int, int, int, int)),this,SLOT(imageReady(QByteArray &, int, int, int, int)));
    connect(m_ksane, SIGNAL(availableDevices(QList<KSaneWidget::DeviceInfo>)),this, SLOT(availableDevices(QList<KSaneWidget::DeviceInfo>)));
    m_ksane->initGetDeviceList();
    openScanner();
    qDebug() <<"init ok";

    connect(ui->bClose, SIGNAL(clicked()), this, SLOT(close()));
    connect(ui->bSettings, SIGNAL(clicked()), this, SLOT(openSettingsWindow()));

    m_imageViewer = new CImageViewer(m_image);
    ui->tabWidget->addTab(m_imageViewer,tr("Images"));
    qDebug() << "image viewer ok";

    QCoreApplication::setOrganizationName("QScan");
    QCoreApplication::setApplicationName("QScan");
    settings = new QSettings();
    //If first run, open settings window
    if (!readSettings()) {
        openSettingsWindow(true);
    }

   //m_image->addImage(QImage("/home/piotrek/prywatne2/scan/1.png"));
  // m_image->addImage(QImage("/home/piotrek/prywatne2/scan/2.png"));
  // m_image->addImage("/home/piotrek/image-.jpg");

}

MainWindow::~MainWindow()
{
    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    m_image->~CImage();
    m_imageThread->deleteLater();
    delete ui;
    QApplication::restoreOverrideCursor();
}

void MainWindow::imageReady(QByteArray &image, int width, int height, int bytespl, int format)
{
    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    bool ok = m_image->addImage(m_ksane->toQImage(image, width, height, bytespl, (KSaneIface::KSaneWidget::ImageFormat)format));
    QApplication::restoreOverrideCursor();
    if (!ok) {
        QMessageBox::critical(this, tr("Unable to save image"), tr("Try to chage default folder."));
        qDebug() << "ERROR: unable to save image" ;
    }
}

void MainWindow::availableDevices(QList<KSaneWidget::DeviceInfo> deviceList)
{
    if(!deviceList.isEmpty()) {
        qDebug() << "Scan device found";
    }
}

void MainWindow::closeEvent(QCloseEvent *event)
{
    writeSettings();
    m_ksane->closeDevice();
    event->accept();
}

void MainWindow::openScanner()
{
    QString dev = m_ksane->selectDevice(this);
    if (dev.isEmpty()) {
        qDebug() << "No scanners found.";
        return;
    }
    //open selected scanner
    if (m_ksane->openDevice(dev)) {
        qDebug() << "device found:" << m_ksane->vendor() << m_ksane->model();
    } else {
        QString scaner = m_ksane->vendor() + " " + m_ksane->model();
        QMessageBox::critical(this, tr("Unable to open scanner"), tr("Opening ") + scaner + tr(" failed."));
        qDebug() << "Unable to open selected scanner: " << scaner ;
    }
}

void MainWindow::writeSettings()
{
    settings->setValue(QString("FirstRun"), QVariant(false));
    settings->setValue(QString("WindowsSize"), QVariant(size()));
    settings->endGroup();
    settings->beginGroup("FileList");
    settings->remove("");
    QStringList list = m_imageViewer->openedImages();
    for (int i=0; i < list.count(); i++) {
        QString name = QString("file%1").arg(i);
        settings->setValue(name, QVariant(list.at(i)));

    }
    settings->endGroup();
}

bool MainWindow::readSettings()
{
    //check if it is first run
    if (!settings->contains("FirstRun"))
        return false;

    //read settings
    resize(settings->value("WindowsSize",QVariant(QSize(500, 400))).toSize());
    //1. dir Name, 2. extension, 3. baseName, 4.start nr
    QList<QVariant> sets;
    sets.append(settings->value("DirPath", QVariant(QDir::homePath() + "/QScanner")));
    sets.append(settings->value("ImageFormat", QVariant("png")));
    sets.append(settings->value("FileNameBase", QVariant("image")));
    sets.append(settings->value("FileStartNumber", QVariant(1)));
    m_image->setSettings(sets);

    settings->beginGroup("FileList");
    QStringList list = settings->childKeys();
    for (int i=0; i < list.count(); i++) {
        m_image->addImage(settings->value(list.at(i)).toString());
    }
    settings->endGroup();
    return true;
}

void MainWindow::openSettingsWindow(bool firstRun)
{
    CSettingsDialog *dialog = new CSettingsDialog(firstRun, this);
    dialog->setPath(settings->value("DirPath").toString());
    dialog->setImageFormat(settings->value("ImageFormat").toString());
    dialog->setFileNameBase(settings->value("FileNameBase").toString());
    dialog->setStartNumber(settings->value("FileStartNumber").toInt());
    if (dialog->exec()) {
        m_image->setSettings(dialog->settingsList());
        settings->setValue("DirPath", QVariant(dialog->path()));
        settings->setValue("ImageFormat", QVariant(dialog->imageFormat()));
        settings->setValue("FileNameBase", QVariant(dialog->fileNameBase()));
        settings->setValue("FileStartNumber", QVariant(dialog->startNumber()));
    }
}
